SystemOrganization addCategory: #'Alien-Core'!
SystemOrganization addCategory: #'Alien-CoreTest'!
SystemOrganization addCategory: #'Alien-Win32'!
(PackageInfo named: 'Alien') postscript: '"Of course this is inadequate.  We need to change the definition of
 recreateSpecialObjectsArray for this setting to persist.  But it at least
 documents the required callback selector for the new facilities."
(SmalltalkImage current specialObjectsArray at: 54) ~= #invokeCallbackContext: ifTrue:
	[SmalltalkImage current specialObjectsArray
		at: 54
		put: #invokeCallbackContext:]'!

!UndefinedObject methodsFor: '*Alien-Core' stamp: 'eem 1/25/2008 13:10'!
asAlien
	^0! !

!WeakArray class methodsFor: '*Alien-Core' stamp: 'eem 2/20/2008 13:57'!
hasWeakDependent: anObject
	^FinalizationDependents includes: anObject! !

!Integer methodsFor: '*Alien-Core' stamp: 'eem 1/16/2008 13:57'!
| anInteger
	^self bitOr: anInteger! !

!True methodsFor: '*Alien-Core' stamp: 'eem 1/25/2008 13:09'!
asAlien

	^ 1! !

!String methodsFor: '*Alien-Core' stamp: 'eem 2/21/2008 15:45'!
asAlien
	| bs |
	^(Alien newC: (bs := self byteSize) + 1)
		replaceFrom: 1 to: bs with: self startingAt: 1! !

ByteArray variableByteSubclass: #Alien
	instanceVariableNames: ''
	classVariableNames: 'GCMallocedAliens LoadedLibraries ProcessIoEvents'
	poolDictionaries: ''
	category: 'Alien-Core'!

!Alien commentStamp: 'John M McIntosh 11/23/2008 12:45' prior: 0!
Copyright 2008 Cadence Design Systems, Inc.
   
   Licensed under the Apache License, Version 2.0 (the ''License''); you may not use this file except in compliance with the License.  You may obtain a copy of the License at  http://www.apache.org/licenses/LICENSE-2.0

Instances of Alien represent actual parameters, return results and function pointers in FFI call-outs and call-backs and provide handles on external data.  See NewsqueakIA32ABIPlugin for the VM code that actually implements call-outs and call-backs.

See the class-side examples category for some simple example workspaces.

Aliens represent ABI (C language) data.  They can hold data directly in their bytes or indirectly by pointing to data on the C heap.  Alien instances are at least 5 bytes in length. The first 4 bytes of an Alien hold the size, as a signed integer, of the datum the instance is a proxy for.  If the size is positive then the Alien is "direct" and the actual datum resides in the object itself, starting at the 5th byte.  If the size is negative then the proxy is "indirect", is at least 8 bytes in length and the second 4 bytes hold the address of the datum, which is assumed to be on the C heap.  Any attempt to access data beyond the size will fail.  If the size is zero then the Alien is a pointer, the second 4 bytes hold a pointer, as for "indirect" Aliens, and accessing primitives indirect through the pointer to access data, but no bounds checking is performed.

When Aliens are used as parameters in FFI calls then all are "passed by value", so that e.g. a 4 byte direct alien will have its 4 bytes of data passed, and a 12-byte indirect alien will have the 12 bytes its address references passed.  Pointer aliens will have their 4 byte pointer passed.  So indirect and pointer aliens are equivalent for accessing data but different when passed as parameters, indirect Aliens passing the data and pointer Aliens passing the pointer.

Class Variables:
GCMallocedAliens <AlienWeakTable of <Alien -> Integer>> - weak collection of malloced aliens, used to free malloced memory of Aliens allocated with newGC:
LoadedLibraries <Dictionary of <String -> Alien>> - library name to library handle map!

!Alien class methodsFor: 'primitives-memory management' stamp: 'eem 12/5/2008 09:41'!
Ccalloc: byteSize "<Integer> ^<Integer>"
	<primitive: 'primCalloc' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien class methodsFor: 'primitives-memory management' stamp: 'eem 12/5/2008 09:41'!
Cmalloc: byteSize "<Integer> ^<Integer>"
	<primitive: 'primMalloc' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien class methodsFor: 'instance creation' stamp: 'eem 2/13/2008 15:44'!
atAddress: pointer "<Integer>" "^<Alien>"
	"N.B.  This method assumes one is creating a pointer to already initialized data, hence initiaize is not sent.
	(Perhaps some alternative, eg. initializePointer, should be sent?"
	^(self basicNew: 8)
		sizeFieldPut: self dataSize negated;
		addressFieldPut: pointer;
		yourself! !

!Alien class methodsFor: 'instance creation' stamp: 'eem 2/13/2008 15:45'!
atAddress: address "<Integer>" dataSize: dataByteSize "<Integer>" "^<Alien>"
	"N.B.  This method assumes one is creating a pointer to already initialized data, hence initiaize is not sent.
	(Perhaps some alternative, eg. initializePointer, should be sent?"
	^(self basicNew: 8)
		sizeFieldPut: dataByteSize asInteger abs negated;
		addressFieldPut: address;
		yourself! !

!Alien class methodsFor: 'instance creation' stamp: 'eem 12/3/2008 09:50'!
dataSize "^<Integer>"
	"Answer the number of bytes required to represent an instance of the receiver.
	This excludes the size word's own bytes and is just the size of the data."
	^self subclassResponsibility! !

!Alien class methodsFor: 'class initialization' stamp: 'eem 4/18/2008 15:14'!
ensureInSpecialObjectsArray
	"Alien must be in the specialObjectsArray to enable the Newsqueak FFI.  Because of
	 the bootstrap the specialObjectsArray may already have been partially initialized with
	 the callback selector and to be large enough (as part of loading immutability), but
	 it will be missing Alien until Alien is loaded.  So check if the specialObjectsArray is
	 of the expected size before recreating and slam Alien in if the specialObjectsArray
	 is already large enough."

	| index |
	self ~~ Alien ifTrue: [^self].

	index := 53.
	((Smalltalk includesKey: #ObjectMemory)
	 and: [((Smalltalk at: #ObjectMemory) classPool at: #ClassAlien ifAbsent: []) ~~ (index - 1)]) ifTrue:
		[self error: 'index probably wrong'].

	Smalltalk specialObjectsArray size < index ifTrue:
		[Smalltalk recreateSpecialObjectsArray].
	
	(Smalltalk specialObjectsArray size < index
	 or: [(Smalltalk specialObjectsArray at: index) ~~ nil
		and: [(Smalltalk specialObjectsArray at: index) ~~ self]]) ifTrue:
		[self error: 'SystemDictionary>>recreateSpecialObjectsArray appears incompatible'].

	(Smalltalk specialObjectsArray at: index) == nil ifTrue:
		[Smalltalk specialObjectsArray at: index put: self]! !

!Alien class methodsFor: 'libraries' stamp: 'eem 11/29/2007 10:35'!
ensureLoaded: aLibraryName "<String> ^<Alien>"
	^LoadedLibraries
		at: aLibraryName
		ifAbsentPut: [(Alien new: 4)
						unsignedLongAt: 1
							put: (self primLoadLibrary: aLibraryName);
						yourself]! !

!Alien class methodsFor: 'class initialization' stamp: 'eem 1/7/2008 19:33'!
ensureNotifiedAtStartup
	"Alien needs to be notified at system startup to clear the LoadedLibraries table
	and otherwise sanitise Aliens with dangling pointers."
	self ~~ Alien ifTrue: [^self].
	Smalltalk addToStartUpList: self after: Delay! !

!Alien class methodsFor: 'class initialization' stamp: 'eem 2/27/2008 16:44'!
ensureNotifiedAtStartupAfter: anotherClass
	"Support routine for putting Alien classes in the start-up list."
	anotherClass ensureNotifiedAtStartup.
	Smalltalk addToStartUpList: self after: anotherClass! !

!Alien class methodsFor: 'examples' stamp: 'kb 9/1/2011 19:16'!
exampleCqsort
	"Call the libc qsort function (which requires a callback)."
	"Alien exampleCqsort"
	"(Time millisecondsToRun: [100 timesRepeat: [Alien exampleCqsort]]) / 100.0"
	| cb rand nElements sizeofDouble values orig sort |
	rand := Random new.
	values := Alien newC: (nElements := 100) * (sizeofDouble := 8).
	1 to: values dataSize by: sizeofDouble do:
		[:i| values doubleAt: i put: rand next].
	orig := (1 to: values dataSize by: sizeofDouble) collect: [:i| values doubleAt: i].
	cb := Callback
			signature:  'int (*)(const void *, const void *)'
			block: [ :arg1 :arg2 | ((arg1 doubleAt: 1) - (arg2 doubleAt: 1)) sign ].
	(self lookup: 'qsort' inLibrary: self libcName)
		primFFICallResult: nil
		with: values pointer
		with: nElements
		with: sizeofDouble
		with: cb thunk.
	sort := (1 to: values dataSize by: sizeofDouble) collect: [:i| values doubleAt: i].
	values free.
	^orig -> sort! !

!Alien class methodsFor: 'examples' stamp: 'eem 2/26/2008 18:39'!
exampleEnumFontFamiliesEx
	"Call the Win32 EnumFontFamiliesEx function."
	"Alien exampleEnumFontFamiliesEx"
	"(Time millisecondsToRun: [1000 timesRepeat: [Alien exampleEnumFontFamiliesEx]]) / 1000.0"
	| fontNames r gle err hDC cb |
	fontNames := OrderedCollection new.
	gle := self lookup: 'GetLastError' inLibrary: 'kernel32.dll'.
	hDC := (self lookup: 'GetDC' inLibrary: 'user32.dll')
				primFFICallResult: (Alien new: 4)
				with: 0. "entire screen"
	err := Alien new: 4.
	cb := Callback
				block: [:args :result|
						fontNames addLast: args logicalFontData elfFullName.
						result returnInteger: 1]
				stdcallArgsClass: EnumFontFamExProc.
	(self lookup: 'EnumFontFamiliesExA' inLibrary: 'gdi32.dll')
		primFFICallResult: (r := Alien new: 4)
		with: hDC
		with: (LOGFONTA newC lfCharset: LOGFONTA DEFAULTCHARSET; yourself) pointer
		with: cb thunk
		with: 0
		with: 0.
	(r signedLongAt: 1) ~= 1 ifTrue:
		[gle primFFICallResult: err].
	(self lookup: 'ReleaseDC' inLibrary: 'user32.dll')
		primFFICallResult: (Alien new: 4)
		with: 0 "entire screen"
		with: hDC.
	^(err signedLongAt: 1) -> fontNames -> fontNames size! !

!Alien class methodsFor: 'examples' stamp: 'eem 2/26/2008 18:39'!
exampleEnumWindows
	"Call the Win32 EnumWindows function, which takes a callback.
	Count how many windows.  Use an Alien to hold the counter on the C heap.
	Answer the count of windows (number of callbacks)."
	"Alien exampleEnumWindows"
	| counter cb |
	counter := Alien newC: 4.
	cb := Callback
				block: [:args :result| | ptr |
						ptr := Alien forPointer: args lParam.
						ptr signedLongAt: 1 put: (ptr signedLongAt: 1) + 1.
						result returnInteger: (ptr signedLongAt: 1)]
				stdcallArgsClass: EnumWindowsProc.
	^((self lookup: 'EnumWindows' inLibrary: 'user32.dll')
		primFFICallResult: (Alien new: 4)
		with: cb thunk
		with: counter pointer) signedLongAt: 1
! !

!Alien class methodsFor: 'examples' stamp: 'eem 2/26/2008 18:40'!
exampleEnumWindowsCallbackPerformance
	"Measure the speed of callbacks using the Win32 EnumWindows function.
	Count how many windows using a Smalltalk counter.
	Answer seconds -> callbacks -> microseconds per callback."
	"Alien exampleEnumWindowsCallbackPerformance"
	| enumWin count ms cb |
	Smalltalk garbageCollect.
	enumWin := self lookup: 'EnumWindows' inLibrary: 'user32.dll'.
	count := 0.
	cb := Callback
			block: [:args :result|
					result returnInteger: (count := count + 1)]
			stdcallArgsClass: EnumWindowsProc.
	ms := Time millisecondsToRun:
			[1000 timesRepeat:
				[enumWin
					primFFICallResult: (Alien new: 4)
					with: cb thunk
					with: 0]].
	^ms / 1000.0 -> count -> (ms * 1000.0 / count)! !

!Alien class methodsFor: 'examples' stamp: 'eem 2/26/2008 18:41'!
exampleEnumWindowsSimple
	"Call the Win32 EnumWindows function, which takes a callback.
	Count how many windows.  Use an Alien to hold the counter on the C heap.
	Answer the count of windows (number of callbacks)."
	"Alien exampleEnumWindowsSimple"
	| count cb |
	count := 0.
	cb := Callback
			block: [:args :result|
					result returnInteger: (count := count + 1)]
			stdcallArgsClass: EnumWindowsProc.
	^((self lookup: 'EnumWindows' inLibrary: 'user32.dll')
		primFFICallResult: (Alien new: 4)
		with: cb thunk
		with: 0) signedLongAt: 1
! !

!Alien class methodsFor: 'examples' stamp: 'eem 2/19/2008 13:44'!
exampleGetOpenFileName
	"Call the Win32 GetOpenFileNameA function."
	"Alien exampleGetOpenFileName"
	| openstruct |
	openstruct := OPENFILENAME newC.
	(self lookup: 'GetOpenFileNameA' inLibrary: 'comdlg32.dll')
		primFFICallResult: nil
		with: openstruct pointer.
	^openstruct fileName! !

!Alien class methodsFor: 'examples' stamp: 'eem 1/3/2008 14:46'!
examplePrintf
	"Call the libc printf function."
	"Alien examplePrintf"
	| r s |
	(self lookup: 'printf' inLibrary: self libcName)
		primFFICallResult: (r := Alien new: 4)
		withArguments: {(s := ('Hello World %d %x !!!!', (String with: Character lf)) asAlien) pointer.
						 123.
						 48879}.
	s free.
	^r signedLongAt: 1! !

!Alien class methodsFor: 'examples' stamp: 'eem 2/26/2008 18:41'!
exampleVarArgsEnumFontFamiliesEx
	"Call the Win32 EnumFontFamiliesEx function."
	"Alien exampleEnumFontFamiliesEx"
	"(Time millisecondsToRun: [1000 timesRepeat: [Alien exampleEnumFontFamiliesEx]]) / 1000.0"
	| fontNames r gle err hDC cb |
	fontNames := OrderedCollection new.
	gle := self lookup: 'GetLastError' inLibrary: 'kernel32.dll'.
	hDC := (self lookup: 'GetDC' inLibrary: 'user32.dll')
				primFFICallResult: (Alien new: 4)
				with: 0. "entire screen"
	err := Alien new: 4.
	cb := Callback
			block: [:args :result|
					fontNames addLast: args logicalFontData elfFullName.
					result returnInteger: 1]
			stdcallArgsClass: EnumFontFamExProc.
	(self lookup: 'EnumFontFamiliesExA' inLibrary: 'gdi32.dll')
		primFFICallResult: (r := Alien new: 4)
		withArguments: {hDC.
						 (LOGFONTA newC lfCharset: LOGFONTA DEFAULTCHARSET; yourself) pointer.
						 cb thunk.
						 0.
						 0}.
	(r signedLongAt: 1) ~= 1 ifTrue:
		[gle primFFICallResult: err].
	(self lookup: 'ReleaseDC' inLibrary: 'user32.dll')
		primFFICallResult: (Alien new: 4)
		with: 0 "entire screen"
		with: hDC.
	^(err signedLongAt: 1) -> fontNames -> fontNames size! !

!Alien class methodsFor: 'finalization' stamp: 'eem 12/21/2007 15:15'!
finalize: mallocTag
	self primFree: mallocTag! !

!Alien class methodsFor: 'instance creation' stamp: 'eem 2/13/2008 15:47'!
forPointer: address "<Integer>" "^<Alien>"
	"N.B.  This method assumes one is creating a pointer to already initialized data, hence initiaize is not sent.
	(Perhaps some alternative, eg. initializePointer, should be sent?"
	^(self basicNew: 8)
		sizeFieldPut: 0;
		addressFieldPut: address;
		yourself! !

!Alien class methodsFor: 'class initialization' stamp: 'eem 2/27/2008 16:46'!
initialize
	"Alien initialize"
	LoadedLibraries isNil ifTrue:
		[LoadedLibraries := Dictionary new].
	GCMallocedAliens isNil ifTrue:
		[GCMallocedAliens := AlienWeakTable newForOwner: self].
	self ensureInSpecialObjectsArray.
	self ensureNotifiedAtStartup! !

!Alien class methodsFor: 'callbacks' stamp: 'vb 5/9/2008 21:43'!
invokeCallback: thunkPtr "<Integer>" stack: stackPtr "<Integer>" registers: regsPtr "<Integer>" jmpbuf: jmpBufPtr "<Integer>" "^<FFICallbackReturnValue>"
	"The low-level entry-point for callbacks sent from the VM/IA32ABI plugin.
	 Return via primReturnFromContext:through:.  thisContext's sender is the
	 call-out context."
	| resultProxy |
	[resultProxy := Callback evaluateCallbackForThunk: thunkPtr stack: stackPtr]
		ifCurtailed: [self error: 'attempt to non-local return across a callback'].
	resultProxy ifNil:
		[(resultProxy := FFICallbackReturnValue new) returnInteger: 0].
	resultProxy primReturnFromContext: thisContext through: jmpBufPtr! !

!Alien class methodsFor: 'callbacks' stamp: 'eem 10/15/2010 10:45'!
invokeCallbackContext: vmCallbackContextAddress "<Integer>" "^<FFICallbackReturnValue>"
	"The low-level entry-point for callbacks sent from the VM/IA32ABI plugin.
	 Return via primReturnFromContext:through:.  thisContext's sender is the
	 call-out context."
	| callbackAlien type |
	callbackAlien := (Smalltalk wordSize = 4
						ifTrue: [VMCallbackContext32]
						ifFalse: [VMCallbackContext64])
							atAddress: vmCallbackContextAddress.
	[type := Callback evaluateCallbackForContext: callbackAlien]
		ifCurtailed: [self error: 'attempt to non-local return across a callback'].
	type ifNil:
		[type := 1. callbackAlien wordResult: -1].
	callbackAlien primReturnAs: type fromContext: thisContext! !

!Alien class methodsFor: 'examples' stamp: 'eem 5/14/2012 15:16'!
libcName
	| platform |
	platform := SmalltalkImage current platformName.
	platform = 'Mac OS' ifTrue:
		[SmalltalkImage current osVersion >= 1050
			ifTrue: ['libgcc_s.1.dylib']
			ifFalse: ['libc.dylib']].
	platform = 'unix' ifTrue:
		[SmalltalkImage current osVersion = 'linux-gnu' ifTrue:
			[^'libc.so.6'].
		 ^'libc.so'].
	platform = 'Win32' ifTrue:
		[^'msvcrt.dll'].
	^nil! !

!Alien class methodsFor: 'libraries' stamp: 'eem 2/12/2008 16:51'!
lookup: symbol  "<String>" inLibrary: libraryName "<String> ^<Alien>"
	^self forPointer: ((self ensureLoaded: libraryName) primFindSymbol: symbol)! !

!Alien class methodsFor: 'libraries' stamp: 'eem 1/11/2008 09:06'!
lookupOrNil: symbol  "<String>" inLibrary: libraryName "<String> ^<Alien>"
	"Answer the address of symbol in libraryName, or nil if it is not in the library.
	Exceptions will be raised for invalid libraries, symbol names, etc."
	^[self lookup: symbol inLibrary: libraryName ]
		on: NotFoundError
		do: [:ex| nil]! !

!Alien class methodsFor: 'instance creation' stamp: 'eem 11/27/2007 10:47'!
new
	"Answer a new instance of the default size whose data resides on the Smalltalk heap directly in the instance."
	^self new: self dataSize! !

!Alien class methodsFor: 'instance creation' stamp: 'eem 11/11/2007 11:40'!
new: byteSize
	"Answer a new instance of the given byte size whose data resides on trhe Smalltalk heap directly in the instance."
	^(self basicNew: byteSize + 4)
		sizeFieldPut: byteSize;
		initialize! !

!Alien class methodsFor: 'instance creation' stamp: 'eem 11/6/2007 12:13'!
newC
	"Answer a new instance of the default size whose data resides on the external C heap."
	^self newC: self dataSize! !

!Alien class methodsFor: 'instance creation' stamp: 'eem 11/12/2007 11:50'!
newC: byteSize
	"Answer a new instance of the given byte size whose data resides on the external C heap.
	 As with instances residing on the Smalltalk heap the data is zero-filled."
	^(self basicNew: 8)
		sizeFieldPut: byteSize negated;
		addressFieldPut: (self Ccalloc: byteSize);
		initialize! !

!Alien class methodsFor: 'instance creation' stamp: 'vb 12/6/2007 11:11'!
newCString: aString "<String> ^<Alien>"
	"Answer an Alien pointing to a null-terminated copy of aString on the external C heap."

	| size cString |
	size := aString size.
	^(Alien rawNewC: size + 1)
		replaceFrom: 1 to: size with: aString startingAt: 1;
		unsignedByteAt: size + 1 put: 0;
		yourself! !

!Alien class methodsFor: 'instance creation' stamp: 'eem 12/2/2008 12:01'!
newGC
	"Answer a new instance of the default size whose data resides on the external C heap,
	 and whose data will be reclaimed automatically when the instance is garbage collected."
	^self newGC: self dataSize! !

!Alien class methodsFor: 'instance creation' stamp: 'eem 12/2/2008 12:01'!
newGC: byteSize
	"Answer a new instance of the given byte size whose data resides on the external C heap,
	 and whose data will be reclaimed automatically when the instance is garbage collected.
	 As with instances residing on the Smalltalk heap the data is zero-filled."
	| alien address |
	(alien := self basicNew: 8)
		sizeFieldPut: byteSize negated.
	address := self Ccalloc: byteSize.
	GCMallocedAliens add: alien finalizing: address.
	^alien
		addressFieldPut: address;
		initialize! !

!Alien class methodsFor: 'primitives-memory management' stamp: 'eem 12/5/2008 09:41'!
primFree: address "<Integer>"
	<primitive: 'primFree' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien class methodsFor: 'primitives-library loading' stamp: 'eem 12/5/2008 09:41'!
primInLibrary: libraryHandle "<Alien>" findSymbol: symbolName "<String> ^<Integer>"
	<primitive: 'primInLibraryFindSymbol' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien class methodsFor: 'primitives-library loading' stamp: 'eem 12/5/2008 09:41'!
primLoadLibrary: libraryName "<String> ^<Integer>"
	"Attempt to load the shared library/DLL named libraryName and
	 answer a handle which can be used in primFindSymbol:inLibrary:,
	 or fail if libraryName is invalid or libraryName cannot be found."
	<primitive: 'primLoadLibrary' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien class methodsFor: 'instance creation' stamp: 'eem 11/12/2007 11:51'!
rawNewC: byteSize
	"Answer a new instance of the given byte size whose data resides on the external C heap.
	 Unlike newC: the data is *not* zero-filled."
	^(self basicNew: 8)
		sizeFieldPut: byteSize negated;
		addressFieldPut: (self Cmalloc: byteSize);
		initialize! !

!Alien class methodsFor: 'system startup' stamp: 'eem 8/18/2011 10:30'!
startUp: resuming
	resuming ifTrue:
		[LoadedLibraries := Dictionary new: LoadedLibraries size.
		"Any addresses in the finalization registry at the time the image
		 comes up in a new session MUST NOT be finalized and should
		 immediately be discarded. These addresses are no longer valid."
		 GCMallocedAliens ifNotNil:
			[:gcMallocedAliens| gcMallocedAliens removeAll].
		"The io lock is off in the new session, so we shut down the process if it's running."
		ProcessIoEvents := false]! !

!Alien methodsFor: 'accessing' stamp: 'eem 2/5/2008 16:27'!
address
	^self sizeField <= 0
		ifTrue: [self addressField]
		ifFalse: [self error: 'One cannot take the address of direct Alien instances; they may move']! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 12/5/2008 09:41'!
addressField "^<Integer>"
	<primitive: 'primAddressField' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 12/5/2008 09:41'!
addressFieldPut: value "<Integer> ^<Integer>"
	<primitive: 'primAddressFieldPut' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'coercing' stamp: 'eem 1/10/2008 22:01'!
asAlien
	^self! !

!Alien methodsFor: 'converting' stamp: 'vb 12/6/2007 11:24'!
asSignedByte "^<Integer>"
	"Answer an Integer created from the first byte of the receiver interpreted as a signed value."
	
	^self signedByteAt: 1! !

!Alien methodsFor: 'converting' stamp: 'vb 12/6/2007 11:24'!
asSignedLong "^<Integer>"
	"Answer an Integer created from the first 4 bytes of the receiver interpreted as a signed value."
	
	^self signedLongAt: 1! !

!Alien methodsFor: 'converting' stamp: 'vb 12/6/2007 11:24'!
asSignedShort "^<Integer>"
	"Answer an Integer created from the first 2 bytes of the receiver interpreted as a signed value."
	
	^self signedShortAt: 1! !

!Alien methodsFor: 'converting' stamp: 'vb 12/6/2007 11:24'!
asUnsignedByte "^<Integer>"
	"Answer an Integer created from the first byte of the receiver interpreted as an unsigned value."
	
	^self unsignedByteAt: 1! !

!Alien methodsFor: 'converting' stamp: 'vb 12/6/2007 11:24'!
asUnsignedLong "^<Integer>"
	"Answer an Integer created from the first 4 bytes of the receiver interpreted as an unsigned value."
	
	^self unsignedLongAt: 1! !

!Alien methodsFor: 'converting' stamp: 'vb 12/6/2007 11:24'!
asUnsignedShort "^<Integer>"
	"Answer an Integer created from the first 2 bytes of the receiver interpreted as an unsigned value."
	
	^self unsignedShortAt: 1! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 12/5/2008 09:41'!
copyInto: aByteObject "<Alien | indexableByteSubclass>" from: start "<Integer>" to: stop "<Integer>" in: replacement "<Alien | indexableByteSubclass>" startingAt: repStart "<Integer>" "^<self>"
	<primitive: 'primAlienReplace' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'accessing' stamp: 'eem 12/20/2007 17:41'!
dataSize
	^self sizeField abs! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 12/5/2008 09:41'!
doubleAt: index "<Integer> ^<Float>"
	<primitive: 'primDoubleAt' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 12/5/2008 09:41'!
doubleAt: index "<Integer>" put: value "<Float | Integer> ^<Float | Integer>"
	<primitive: 'primDoubleAtPut' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 12/5/2008 09:41'!
floatAt: index "<Integer> ^<Float>"
	<primitive: 'primFloatAt' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 12/5/2008 09:41'!
floatAt: index "<Integer>" put: value "<Float | Integer> ^<Float | Integer>"
	<primitive: 'primFloatAtPut' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'memory management' stamp: 'eem 12/21/2007 15:50'!
free
	self primFree! !

!Alien methodsFor: 'accessing' stamp: 'eem 4/18/2008 15:14'!
freeAfter: aBlock
	"The receiver must be a pointer to a previously allocated external C heap memory block. Invoke aBlock passing the receiver as the argument, then free the receiver. Allows automatic allocation and freeing of C artifacts using patters such as
	(Alien newCString: 'foo') freeAfter: [:cstring | ...use cstring here...]."
	
	^[aBlock value: self] ensure: [self free]! !

!Alien methodsFor: 'instance initalization' stamp: 'eem 11/6/2007 12:09'!
initialize
	"Subclasses should redefine this method to perform initializations on instance creation"
	"Overrides ProtoObjects' method for explicitness."! !

!Alien methodsFor: 'accessing' stamp: 'eem 2/5/2008 16:27'!
pointer
	"Answer a pointer to the receiver in the form of an Alien.
	 This will fail if the receiver is not indirect."
	self sizeField = 0 ifTrue:
		[self error: 'One cannot take the address of pointer Alien instances; they may move, and are already pointers'].
	^self class forPointer: self address! !

!Alien methodsFor: 'primitives-callouts' stamp: 'eem 12/5/2008 09:41'!
primFFICallResult: result "<Alien> ^<Alien>"
	<primitive: 'primCallOutIntegralReturn' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'eem 12/5/2008 09:41'!
primFFICallResult: result "<Alien>" with: firstArg "<Alien | Integer> ^<Alien>"
	<primitive: 'primCallOutIntegralReturn' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'eem 12/5/2008 09:41'!
primFFICallResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer> ^<Alien>"
	<primitive: 'primCallOutIntegralReturn' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'eem 12/5/2008 09:41'!
primFFICallResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | SmallInteger> ^<Alien>"
	<primitive: 'primCallOutIntegralReturn' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'eem 12/5/2008 09:41'!
primFFICallResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer> ^<Alien>"
	<primitive: 'primCallOutIntegralReturn' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'eem 12/5/2008 09:41'!
primFFICallResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer> ^<Alien>"
	<primitive: 'primCallOutIntegralReturn' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'eem 12/5/2008 09:41'!
primFFICallResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer> ^<Alien>"
	<primitive: 'primCallOutIntegralReturn' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'eem 12/5/2008 09:41'!
primFFICallResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer>" with: arg7 "<Alien | Integer>" "^<Alien>"
	<primitive: 'primCallOutIntegralReturn' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'eem 12/5/2008 09:41'!
primFFICallResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer>" with: arg7 "<Alien | Integer>" with: arg8 "<Alien | Integer>" "^<Alien>"
	<primitive: 'primCallOutIntegralReturn' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'eem 12/5/2008 09:41'!
primFFICallResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer>" with: arg7 "<Alien | Integer>" with: arg8 "<Alien | Integer>" with: arg9 "<Alien | Integer>" "^<Alien>"
	<primitive: 'primCallOutIntegralReturn' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'eem 12/5/2008 09:41'!
primFFICallResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer>" with: arg7 "<Alien | Integer>" with: arg8 "<Alien | Integer>" with: arg9 "<Alien | Integer>" with: arg10 "<Alien | Integer>" "^<Alien>"
	<primitive: 'primCallOutIntegralReturn' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'eem 12/5/2008 09:41'!
primFFICallResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer>" with: arg7 "<Alien | Integer>" with: arg8 "<Alien | Integer>" with: arg9 "<Alien | Integer>" with: arg10 "<Alien | Integer>" with: arg11 "<Alien | Integer>" "^<Alien>"
	<primitive: 'primCallOutIntegralReturn' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'eem 12/5/2008 09:41'!
primFFICallResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer>" with: arg7 "<Alien | Integer>" with: arg8 "<Alien | Integer>" with: arg9 "<Alien | Integer>" with: arg10 "<Alien | Integer>" with: arg11 "<Alien | Integer>" with: arg12 "<Alien | Integer>" "^<Alien>"
	<primitive: 'primCallOutIntegralReturn' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'eem 12/5/2008 09:41'!
primFFICallResult: result "<Alien>" withArguments: args "<Array of: Alien | Integer> ^<Alien>"
	<primitive: 'primVarArgsCallOutIntegralReturn' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-library loading' stamp: 'eem 12/5/2008 09:41'!
primFindSymbol: symbolName "<String> ^<Integer>"
	"The receiver is assumed to be a proxy whose first field contans the handle of a library
	 answered by primLoadLibrary.  Attempt to find symbolName in the library and answer
	 its address, or fail if the receiver is invalid or the symbol cannot be found."
	<primitive: 'primInLibraryFindSymbol' module: 'IA32ABI' error: errorCode>
	^errorCode == #'not found'
		ifTrue: [NotFoundError signal]
		ifFalse: [self primitiveFailed]! !

!Alien methodsFor: 'primitives-memory management' stamp: 'eem 12/5/2008 09:41'!
primFree
	"Free the memory referenced by the receiver, or fail if
	 the receiver is direct or the indirect pointer is invalid.
	 Note that if successful the address and size fields are
	 zero'ed, effectively preventing double-freeing hazards."
	<primitive: 'primBoxedFree' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 12/5/2008 09:41'!
replaceFrom: start "<Integer>" to: stop "<Integer>" with: replacement "<Alien | indexableByteSubclass>" startingAt: repStart "<Integer>" "^<self>"
	<primitive: 'primAlienReplace' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 12/5/2008 09:41'!
signedByteAt: index "<Integer> ^<Integer>"
	<primitive: 'primSignedByteAt' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 12/5/2008 09:41'!
signedByteAt: index "<Integer>" put: value "<Integer> ^<Integer>"
	<primitive: 'primSignedByteAtPut' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 12/5/2008 09:41'!
signedLongAt: index "<Integer> ^<Integer>"
	<primitive: 'primSignedLongAt' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 12/5/2008 09:41'!
signedLongAt: index "<Integer>" put: value "<Integer> ^<Integer>"
	<primitive: 'primSignedLongAtPut' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'John M McIntosh 12/5/2008 09:41'!
signedLongLongAt: index "<Integer> ^<Integer>"
	<primitive: 'primSignedLongLongAt' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'John M McIntosh 12/5/2008 09:41'!
signedLongLongAt: index "<Integer>" put: value "<Integer> ^<Integer>"
	<primitive: 'primSignedLongLongAtPut' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 12/5/2008 09:41'!
signedShortAt: index "<Integer> ^<Integer>"
	<primitive: 'primSignedShortAt' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 12/5/2008 09:41'!
signedShortAt: index "<Integer>" put: value "<Integer> ^<Integer>"
	<primitive: 'primSignedShortAtPut' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 12/5/2008 09:41'!
sizeField "^<Integer>"
	<primitive: 'primSizeField' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 12/5/2008 09:41'!
sizeFieldPut: size "<Integer> ^<Integer>"
	<primitive: 'primSizeFieldPut' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'printing' stamp: 'eem 10/14/2010 16:39'!
storeOn: aStream
	| sizeField |
	(sizeField := self sizeField) > 0	ifTrue:
		[aStream nextPut: $(].
	aStream nextPut: $(; print: self class.
	sizeField = 0
		ifTrue:
			[aStream
				nextPutAll: ' forPointer: '; nextPutAll: self addressField storeStringHex]
		ifFalse:
			[aStream
				nextPutAll: ' basicNew: '; print: sizeField + 4; nextPut: $);
				crtab; nextPutAll: 'sizeFieldPut: '; print: sizeField.
			 sizeField > 0 ifTrue:
				[1 to: sizeField by: 4 do:
					[:i| | v |
					(v := self unsignedLongAt: i) ~= 0 ifTrue:
						[aStream
							nextPut: $;; crtab;
							nextPutAll: 'unsignedLongAt: '; print: i;
							nextPutAll: ' put: '; print: v]]].
				aStream nextPut: $;; crtab; nextPutAll: #yourself].
	aStream nextPut: $)! !

!Alien methodsFor: 'accessing' stamp: 'eem 2/4/2008 16:29'!
strcpy "^<ByteString>"
	^self strcpyFrom: 1! !

!Alien methodsFor: 'accessing' stamp: 'eem 11/29/2007 12:00'!
strcpyFrom: startIndex "<Integer> ^<ByteString>"
	| length string |
	length := self strlenStartingAt: startIndex.
	self copyInto: (string := ByteString new: length)
		from: 1 to: length
		in: self startingAt: startIndex.
	^string! !

!Alien methodsFor: 'accessing' stamp: 'eem 2/4/2008 16:29'!
strcpyUTF8 "^<String>"
	^self strcpyUTF8From: 1! !

!Alien methodsFor: 'accessing' stamp: 'John M McIntosh 11/23/2008 23:41'!
strcpyUTF8From: startIndex "<Integer> ^<String>"
	| length bytes |
	length := self strlenStartingAt: startIndex.
	self copyInto: (bytes := ByteArray new: length)
		from: 1 to: length
		in: self startingAt: startIndex.
	^bytes utf8ToIso! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 12/5/2008 09:41'!
strlenStartingAt: index "<Integer> ^<Integer>"
	<primitive: 'primStrlenFromStartIndex' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 12/5/2008 09:41'!
strlenThroughPointerAt: index "<Integer> ^<Integer>"
	<primitive: 'primStrlenThroughPointerAtIndex' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 12/5/2008 09:41'!
unsignedByteAt: index "<Integer> ^<Integer>"
	<primitive: 'primUnsignedByteAt' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 12/5/2008 09:41'!
unsignedByteAt: index "<Integer>" put: value "<Integer> ^<Integer>"
	<primitive: 'primUnsignedByteAtPut' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 12/5/2008 09:41'!
unsignedLongAt: index "<Integer> ^<Integer>"
	<primitive: 'primUnsignedLongAt' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 12/5/2008 09:41'!
unsignedLongAt: index "<Integer>" put: value "<Integer> ^<Integer>"
	<primitive: 'primUnsignedLongAtPut' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'John M McIntosh 12/5/2008 09:41'!
unsignedLongLongAt: index "<Integer> ^<Integer>"
	<primitive: 'primUnsignedLongLongAt' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'John M McIntosh 12/5/2008 09:41'!
unsignedLongLongAt: index "<Integer>" put: value "<Integer> ^<Integer>"
	<primitive: 'primUnsignedLongLongAtPut' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 12/5/2008 09:41'!
unsignedShortAt: index "<Integer> ^<Integer>"
	<primitive: 'primUnsignedShortAt' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 12/5/2008 09:41'!
unsignedShortAt: index "<Integer>" put: value "<Integer> ^<Integer>"
	<primitive: 'primUnsignedShortAtPut' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

Alien variableByteSubclass: #EnumFontFamExProc
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alien-Win32'!

!EnumFontFamExProc commentStamp: 'eem 11/28/2007 14:00' prior: 0!
int CALLBACK EnumFontFamExProc(
  ENUMLOGFONTEX *lpelfe,    // logical-font data
  NEWTEXTMETRICEX *lpntme,  // physical-font data
  DWORD FontType,           // type of font
  LPARAM lParam             // application-defined data
);
!

!EnumFontFamExProc class methodsFor: 'instance creation' stamp: 'eem 11/28/2007 13:44'!
dataSize
	^16! !

!EnumFontFamExProc methodsFor: 'accessing' stamp: 'eem 11/28/2007 14:34'!
FontType
	"DWORD FontType,           // type of font"
	^self unsignedLongAt: 9! !

!EnumFontFamExProc methodsFor: 'accessing' stamp: 'eem 11/28/2007 14:34'!
lParam
	"LPARAM lParam             // application-defined data"
	^self unsignedLongAt: 13! !

!EnumFontFamExProc methodsFor: 'accessing' stamp: 'eem 1/30/2008 14:30'!
logicalFontData
	"Convenience for ENUMLOGFONTEX *lpelfe,    // logical-font data"
	^ENUMLOGFONTEXA atAddress: (self unsignedLongAt: 1)! !

!EnumFontFamExProc methodsFor: 'accessing' stamp: 'eem 11/28/2007 14:30'!
lpelfe
	"ENUMLOGFONTEX *lpelfe,    // logical-font data"
	^self unsignedLongAt: 1! !

!EnumFontFamExProc methodsFor: 'accessing' stamp: 'eem 11/28/2007 13:59'!
lpntme
	"NEWTEXTMETRICEX *lpntme,  // physical-font data"
	^self unsignedLongAt: 5! !

Alien variableByteSubclass: #EnumWindowsProc
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alien-Win32'!

!EnumWindowsProc commentStamp: 'eem 11/28/2007 14:28' prior: 0!
BOOL CALLBACK EnumWindowsProc(HWND hwnd, LPARAM lParam);
!

!EnumWindowsProc class methodsFor: 'instance creation' stamp: 'eem 11/19/2007 16:36'!
dataSize
	^8! !

!EnumWindowsProc methodsFor: 'accessing' stamp: 'eem 11/28/2007 14:28'!
hwnd
	"BOOL CALLBACK EnumWindowsProc(HWND hwnd, LPARAM lParam);"
	^self unsignedLongAt: 1! !

!EnumWindowsProc methodsFor: 'accessing' stamp: 'eem 11/28/2007 14:28'!
lParam
	"BOOL CALLBACK EnumWindowsProc(HWND hwnd, LPARAM lParam);"
	^self unsignedLongAt: 5! !

Alien variableByteSubclass: #FFICallbackReturnValue
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alien-Core'!

!FFICallbackReturnValue commentStamp: 'eem 4/14/2008 14:19' prior: 0!
An instance of FFICallbackReturnValue specifies a return value to be passed to a callback callee.  It is intended to have overlaid the following struct:
/*
 * Returning values from callbacks is done through a CallBackReturnSpec
 * which contains a type tag and values.  It is designed to be overlaid upon
 * an FFICallbackReturnProxy created at the Smalltalk level to return values.
 */
typedef struct {
    long type;
# define retint32  0 
# define retint64  1
# define retdouble 2
# define retstruct 3
    long _pad; /* so no doubt that valflt64 & valint32 et al are at byte 8 */
    union {
        long valint32;
        struct { long low, high; } valint64;
        double valflt64;
        struct { void *addr; long size; } valstruct;
    } rvs;
} CallBackReturnSpec;
!

!FFICallbackReturnValue class methodsFor: 'instance creation' stamp: 'eem 4/14/2008 14:18'!
dataSize
	"See class comment"
	^16! !

!FFICallbackReturnValue methodsFor: 'primitives-callbacks' stamp: 'eem 12/5/2008 09:41'!
primReturnFromContext: callbackContext "<ContextPart>" through: jmpbuf "<Integer>" "^not reached"
	"Return a value, as described by the receiver, to some C callee that has made a
	callback.   Reestablish the interpreter's state prior to the callback so that either
	the callout can return or subsequent callbacks can occur.  callbackContext is an
	activation of invokeCallback:stack:registers:jmpbuf:.  Its sender is the interpreter's
	state prior to the callback"
	<primitive: 'primReturnFromContextThrough' module: 'IA32ABI' error: errorCode>
	self primitiveFailed! !

!FFICallbackReturnValue methodsFor: 'instance initalization' stamp: 'eem 11/19/2007 16:27'!
returnDouble: f
	self signedLongAt: self typeOffset put: 2.
	self doubleAt: self valueOffset put: f asFloat! !

!FFICallbackReturnValue methodsFor: 'instance initalization' stamp: 'eem 11/19/2007 16:27'!
returnInteger: n
	self signedLongAt: self typeOffset put: 0.
	self signedLongAt: self valueOffset put: n asInteger! !

!FFICallbackReturnValue methodsFor: 'field offsets' stamp: 'eem 11/18/2007 13:45'!
typeOffset
	^1 "0"! !

!FFICallbackReturnValue methodsFor: 'field offsets' stamp: 'eem 4/14/2008 14:19'!
valueOffset
	^9 "8"! !

Alien variableByteSubclass: #FFICallbackThunk
	instanceVariableNames: ''
	classVariableNames: 'AccessProtect AllocatedThunks ExecutablePages'
	poolDictionaries: ''
	category: 'Alien-Core'!

!FFICallbackThunk commentStamp: 'eem 12/21/2007 15:53' prior: 0!
An instance of FFICallbackThunk is a reference to a machine-code thunk/trampoline that calls-back into the VM.  The reference can be passed to C code which can use it as a function pointer through which to call-back into Smalltalk.  The machine-code thunk/trampoline is different for each instance, hence its address is a unique key that can be used to assocuate the Smalltalk side of the call-back (e.g. a block) with the thunk.  Since thunks must be executable and some OSs may not provide default execute permission on memory returned by malloc we may not be able to use malloc directly.  Instead we rely on a primitive to provide memory that is guaranteed to be executable.  The FFICallbackThunk class>>allocateExectablePage primitive answers an Alien that references an executable piece of memory that is some (possiby unitary) multiple of the pagesize.  Class-side code then parcels out pieces of a page to individual thunks.  These pieces are recycled when thunks are reclaimed.  Since the first byte of a thunk is non-zero we can use it as a flag indicating if the piece is in use or not.

See Callback for the higher-level construct that represents a Smalltalk block to be run in response to a callback.  Callbacks wrap instances of FFICallbackThunk and arbitrary Alien instances that describe the stack layout for receiving arguments.

Class Variables
AccessProtect <Semaphore> critical section for ExecutablePages (de)allocation
AllocatedThunks <AlienWeakTable of <FFICallbackThunk -> Integer>> - weak collection of thunks, used to return thunk storage to the executable page pool.
ExecutablePages <Set of: Alien "executable page"> - collection of pages with execute permissions used to provide executable thunks!

!FFICallbackThunk class methodsFor: 'executable page management' stamp: 'eem 12/5/2008 09:41'!
allocateExecutablePage "^<Alien>"
	"Answer an Alien around a piece of (probably malloced) memory, of some multiple
	 of the pagesize, that has execute permissions set.  This memory can be parcelled
	 out to individual FFICallbackThunks to hold their thunk/trampoline machine code."
	<primitive: 'primAllocateExecutablePage' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!FFICallbackThunk class methodsFor: 'instance creation' stamp: 'eem 12/20/2007 17:06'!
dataSize
	^24 "see FFICallbackThunk initialize & initializeStdcall:"! !

!FFICallbackThunk class methodsFor: 'class initialization' stamp: 'eem 2/27/2008 16:45'!
ensureNotifiedAtStartup
	"FFICallbackThunk needs to be notified at system startup to clear the ExecutablePages table."
	self ~~ FFICallbackThunk ifTrue: [^self].
	self ensureNotifiedAtStartupAfter: Alien! !

!FFICallbackThunk class methodsFor: 'finalization' stamp: 'eem 12/21/2007 15:17'!
finalize: address
	self freeBlockInExecutablePage: address! !

!FFICallbackThunk class methodsFor: 'executable page management' stamp: 'eem 12/20/2007 17:41'!
freeBlockInExecutablePage: address
	ExecutablePages do:
		[:alienPage| | alienAddress |
		alienAddress := alienPage address.
		(address >= alienAddress
		 and: [alienAddress + alienPage dataSize >= address]) ifTrue:
			[alienPage unsignedByteAt: address - alienAddress + 1 put: 0.
			 ^self]]
			! !

!FFICallbackThunk class methodsFor: 'class initialization' stamp: 'eem 2/27/2008 16:47'!
initialize
	"FFICallbackThunk initialize"
	AccessProtect := Semaphore forMutualExclusion.
	AllocatedThunks := AlienWeakTable newForOwner: self.
	ExecutablePages := Set new.
	self ensureNotifiedAtStartup! !

!FFICallbackThunk class methodsFor: 'instance creation' stamp: 'eem 12/21/2007 15:32'!
newCCall
	"Answer a new instance for the C calling-convention."
	^self privateFinalizingNew initialize! !

!FFICallbackThunk class methodsFor: 'instance creation' stamp: 'eem 12/21/2007 15:32'!
newStdCall: argBytes
	"Answer a new instance for Windows' StdCall (Pascal) convention, given the
	 size of the arguments in bytes (bytes to cut-back from the stack on return)."
	^self privateFinalizingNew initializeStdcall: argBytes! !

!FFICallbackThunk class methodsFor: 'instance creation' stamp: 'eem 1/12/2010 13:42'!
privateFinalizingNew
	"Answer a new instance that will free its page when garbage collected."
	| address |
	address := self unusedBlockInExecutablePage.
	^AllocatedThunks
		add: (self forPointer: address)
		finalizing: address ! !

!FFICallbackThunk class methodsFor: 'class initialization' stamp: 'vb 5/8/2008 11:32'!
startUp: inANewImageSession
	"Any thunks in the finalization registry at the time the image comes up in a new session MUST NOT be finalized and should  immediately be discarded. Their thunk pointers are no longer valid."
	inANewImageSession ifTrue:
		[AllocatedThunks removeAll.
		ExecutablePages := Set new]! !

!FFICallbackThunk class methodsFor: 'executable page management' stamp: 'eem 12/21/2007 15:28'!
unusedBlockInExecutablePage
	AccessProtect critical:
		[ExecutablePages do:
			[:alien|
			1 to: alien dataSize - self dataSize by: self dataSize do:
				[:i|
				(alien unsignedByteAt: i) = 0 ifTrue:
					[alien unsignedByteAt: i put: 1.
					 ^alien address + i - 1]]]].
	^(AccessProtect critical: [ExecutablePages add: self allocateExecutablePage])
		unsignedByteAt: 1 put: 1;
		address! !

!FFICallbackThunk methodsFor: 'accessing' stamp: 'eem 1/29/2008 13:06'!
address
	"Thunks are hacked - again they need to be indirect objects with a 0 size field."
	^self addressField! !

!FFICallbackThunk methodsFor: 'memory management' stamp: 'eem 12/21/2007 15:38'!
free
	"Recycle the memory in the executable page containing the receiver's thunk.
	 Since the receiver is a derived pointer to a block in an executable page this
	 requires returning the block to the free pool."
	AllocatedThunks remove: self.
	AccessProtect critical:
		[| address |
		address := self addressField.
		address ~= 0 ifTrue:
			[self addressFieldPut: 0.
			 self class freeBlockInExecutablePage: address]]! !

!FFICallbackThunk methodsFor: 'instance initalization' stamp: 'eem 1/12/2010 13:43'!
initialize
	"Initialize the receiver with a __ccall thunk."
	"thunk:		push   %esp					0x54							0xa1905454
	 thunk+01:	push   %esp					0x54
	 thunk+02:	nop   						0x90
	 thunk+03:	mov    $thunkEntry,%eax	0xb8 0x00 0x00 0x00 0x00	0x00000000 - entry
	 thunk+08:	nop   						0x90							0x68909090
	 thunk+09:	nop   						0x90
	 thunk+10:	nop   						0x90
	 thunk+11:	push   $thunk				0x68 0x00 0x00 0x00 0x00	0x00000000 - thunk
	 thunk+16:	call   *%eax					0xff 0xd0						0xc483d0ff
	 thunk+18:	add    $0xC,%esp			0x83 0xc4 0x0C				0x9090c30C
	 thunk+21:	ret							0xc3
	 thunk+22:	nop							0x90
	 thunk+23:	nop							0x90"
	self
		unsignedLongAt:  1 put: 16rB8905454;
		unsignedLongAt:  5 put: self primThunkEntryAddress;
		unsignedLongAt:  9 put: 16r68909090;
		unsignedLongAt: 13 put: self addressField;
		unsignedLongAt: 17 put: 16rC483D0FF;
		unsignedLongAt: 21 put: 16r9090C30C! !

!FFICallbackThunk methodsFor: 'instance initalization' stamp: 'eem 1/12/2010 13:43'!
initializeStdcall: bytes
	"Initialize the receiver with a __stdcall thunk with bytes argument bytes."
	"thunk:		push   %esp					0x54							0xa1905454
	 thunk+01:	push   %esp					0x54
	 thunk+02:	nop   						0x90
	 thunk+03:	mov    $thunkEntry,%eax	0xb8 0x00 0x00 0x00 0x00	0x00000000 - entry
	 thunk+08:	nop   						0x90							0x68909090
	 thunk+09:	nop   						0x90
	 thunk+10:	nop   						0x90
	 thunk+11:	push   $thunk				0x68 0x00 0x00 0x00 0x00	0x00000000 - thunk
	 thunk+16:	call   *%eax					0xff 0xd0						0xc483d0ff
	 thunk+18:	add    $0xC,%esp			0x83 0xc4 0x0C				0xBYTSc20C
	 thunk+21:	ret    $bytes				0xc2 0xBY 0xTS"

	self
		unsignedLongAt:  1 put: 16rB8905454;
		unsignedLongAt:  5 put: self primThunkEntryAddress;
		unsignedLongAt:  9 put: 16r68909090;
		unsignedLongAt: 13 put: self addressField;
		unsignedLongAt:  17 put: 16rC483D0FF;
		unsignedShortAt: 21 put: 16rC20C;
		unsignedShortAt: 23 put: bytes! !

!FFICallbackThunk methodsFor: 'primitives-memory management' stamp: 'eem 12/21/2007 15:45'!
primFree
	self shouldNotImplement! !

!FFICallbackThunk methodsFor: 'private-primitives' stamp: 'eem 12/5/2008 09:41'!
primThunkEntryAddress "^<Integer>"
	"Answer the address of the entry-point for thunk callbacks:
		long thunkEntry(void *thunkp, long *stackp);
	 This is the function a callback thunk/trampoline should call to initiate a callback."
	<primitive: 'primThunkEntryAddress' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

Alien variableByteSubclass: #IntegerFlagAlien
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alien-Win32'!

!IntegerFlagAlien commentStamp: 'eem 4/25/2008 16:30' prior: 0!
A class to wrap the inIoProcessEvents flag in the WIndows VM.!

!IntegerFlagAlien class methodsFor: 'instance creation' stamp: 'eem 4/25/2008 16:34'!
forInIoProcessEventsFlag
	^self
		atAddress: self primInIOProcessEventsFlagAddress
		dataSize: 4! !

!IntegerFlagAlien class methodsFor: 'primitives-Windows-VM-specific' stamp: 'eem 12/5/2008 09:41'!
primInIOProcessEventsFlagAddress "^<Integer>"
	"Answer the address of the Windows-VM-specific int inIoProcessEvents flag.
	 See platforms/win32/vm/sqWin32Window.c
	 and platforms/Cross/plugins/IA32ABI/ia32abicc.c."
	<primitive: 'primInIOProcessEventsFlagAddress' module: 'IA32ABI' error: errorCode>
	^self primitiveFailed! !

!IntegerFlagAlien methodsFor: 'accessing' stamp: 'eem 4/25/2008 16:32'!
decrement
	| value |
	(value := self value) > 0 ifTrue:
		[self value: value - 1]! !

!IntegerFlagAlien methodsFor: 'accessing' stamp: 'eem 4/25/2008 16:31'!
increment
	self value: self value + 1! !

!IntegerFlagAlien methodsFor: 'accessing' stamp: 'eem 4/25/2008 16:31'!
value
	^self signedLongAt: 1! !

!IntegerFlagAlien methodsFor: 'accessing' stamp: 'eem 4/25/2008 16:31'!
value: anInteger
	self signedLongAt: 1 put: anInteger! !

Alien variableByteSubclass: #LOGFONTA
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alien-Win32'!

!LOGFONTA commentStamp: 'eem 11/28/2007 14:32' prior: 0!
#define LF_FACESIZE 32
typedef struct tagLOGFONT { 
  LONG lfHeight; 
  LONG lfWidth; 
  LONG lfEscapement; 
  LONG lfOrientation; 
  LONG lfWeight; 
  BYTE lfItalic; 
  BYTE lfUnderline; 
  BYTE lfStrikeOut; 
  BYTE lfCharSet; 
  BYTE lfOutPrecision; 
  BYTE lfClipPrecision; 
  BYTE lfQuality; 
  BYTE lfPitchAndFamily; 
  TCHAR lfFaceName[LF_FACESIZE]; 
} LOGFONT, *PLOGFONT;!

LOGFONTA variableByteSubclass: #ENUMLOGFONTEXA
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alien-Win32'!

!ENUMLOGFONTEXA commentStamp: 'eem 11/28/2007 14:31' prior: 0!
#define LF_FULLFACESIZE 64
typedef struct tagENUMLOGFONTEX {
  LOGFONT  elfLogFont;
  TCHAR  elfFullName[LF_FULLFACESIZE];
  TCHAR  elfStyle[LF_FACESIZE];
  TCHAR  elfScript[LF_FACESIZE];
} ENUMLOGFONTEX, *LPENUMLOGFONTEX;
!

!ENUMLOGFONTEXA class methodsFor: 'instance creation' stamp: 'eem 11/28/2007 13:49'!
dataSize
	^188
	
"
#define LF_FULLFACESIZE 64
typedef struct tagENUMLOGFONTEX {
  LOGFONT  elfLogFont;
  TCHAR  elfFullName[LF_FULLFACESIZE];
  TCHAR  elfStyle[LF_FACESIZE];
  TCHAR  elfScript[LF_FACESIZE];
} ENUMLOGFONTEX, *LPENUMLOGFONTEX;

"! !

!ENUMLOGFONTEXA methodsFor: 'accessing' stamp: 'eem 11/28/2007 18:06'!
elfFullName
	^self strcpyFrom: 61! !

!LOGFONTA class methodsFor: 'constants' stamp: 'eem 11/28/2007 13:24'!
DEFAULTCHARSET "actually DEFAULT_CHARSET"
	^1
	
"
#define ANSI_CHARSET            0
#define DEFAULT_CHARSET         1
#define SYMBOL_CHARSET          2
#define SHIFTJIS_CHARSET        128
#define HANGEUL_CHARSET         129
#define HANGUL_CHARSET          129
#define GB2312_CHARSET          134
#define CHINESEBIG5_CHARSET     136
#define OEM_CHARSET             255
#define JOHAB_CHARSET           130
#define HEBREW_CHARSET          177
#define ARABIC_CHARSET          178
#define GREEK_CHARSET           161
#define TURKISH_CHARSET         162
#define VIETNAMESE_CHARSET      163
#define THAI_CHARSET            222
#define EASTEUROPE_CHARSET      238
#define RUSSIAN_CHARSET         204
#define MAC_CHARSET             77
#define BALTIC_CHARSET          186
"! !

!LOGFONTA class methodsFor: 'instance creation' stamp: 'eem 11/28/2007 13:20'!
dataSize
	^60

"
#define LF_FACESIZE 32
typedef struct tagLOGFONT { 
  LONG lfHeight; 
  LONG lfWidth; 
  LONG lfEscapement; 
  LONG lfOrientation; 
  LONG lfWeight; 
  BYTE lfItalic; 
  BYTE lfUnderline; 
  BYTE lfStrikeOut; 
  BYTE lfCharSet; 
  BYTE lfOutPrecision; 
  BYTE lfClipPrecision; 
  BYTE lfQuality; 
  BYTE lfPitchAndFamily; 
  TCHAR lfFaceName[LF_FACESIZE]; 
} LOGFONT, *PLOGFONT;"! !

!LOGFONTA methodsFor: 'accessing' stamp: 'eem 11/28/2007 18:03'!
lfCharset "^<Integer>"
	self unsignedByteAt: 24! !

!LOGFONTA methodsFor: 'accessing' stamp: 'eem 11/28/2007 18:03'!
lfCharset: charset "<Integer>"
	self unsignedByteAt: 24 put: charset! !

!LOGFONTA methodsFor: 'accessing' stamp: 'eem 11/29/2007 11:54'!
lfFaceName
	^self strcpyFrom: 29! !

Alien variableByteSubclass: #OPENFILENAME
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alien-Win32'!

!OPENFILENAME commentStamp: 'eem 11/28/2007 14:32' prior: 0!
typedef struct tagOFN { 
  DWORD         lStructSize; 
  HWND          hwndOwner; 
  HINSTANCE     hInstance; 
  LPCTSTR       lpstrFilter; 
  LPTSTR        lpstrCustomFilter; 
  DWORD         nMaxCustFilter; 
  DWORD         nFilterIndex; 
  LPTSTR        lpstrFile; 
  DWORD         nMaxFile; 
  LPTSTR        lpstrFileTitle; 
  DWORD         nMaxFileTitle; 
  LPCTSTR       lpstrInitialDir; 
  LPCTSTR       lpstrTitle; 
  DWORD         Flags; 
  WORD          nFileOffset; 
  WORD          nFileExtension; 
  LPCTSTR       lpstrDefExt; 
  LPARAM        lCustData; 
  LPOFNHOOKPROC lpfnHook; 
  LPCTSTR       lpTemplateName; 
#if (_WIN32_WINNT >= 0x0500)
  void *        pvReserved;
  DWORD         dwReserved;
  DWORD         FlagsEx;
#endif // (_WIN32_WINNT >= 0x0500)
} OPENFILENAME, *LPOPENFILENAME;
!

!OPENFILENAME class methodsFor: 'instance creation' stamp: 'eem 11/28/2007 14:32'!
dataSize
	"sizeof(OPENFILENAME) in Windows 5 (76 in Windows 4)"
	^88
	
"
typedef struct tagOFN { 
  DWORD         lStructSize; 
  HWND          hwndOwner; 
  HINSTANCE     hInstance; 
  LPCTSTR       lpstrFilter; 
  LPTSTR        lpstrCustomFilter; 
  DWORD         nMaxCustFilter; 
  DWORD         nFilterIndex; 
  LPTSTR        lpstrFile; 
  DWORD         nMaxFile; 
  LPTSTR        lpstrFileTitle; 
  DWORD         nMaxFileTitle; 
  LPCTSTR       lpstrInitialDir; 
  LPCTSTR       lpstrTitle; 
  DWORD         Flags; 
  WORD          nFileOffset; 
  WORD          nFileExtension; 
  LPCTSTR       lpstrDefExt; 
  LPARAM        lCustData; 
  LPOFNHOOKPROC lpfnHook; 
  LPCTSTR       lpTemplateName; 
#if (_WIN32_WINNT >= 0x0500)
  void *        pvReserved;
  DWORD         dwReserved;
  DWORD         FlagsEx;
#endif // (_WIN32_WINNT >= 0x0500)
} OPENFILENAME, *LPOPENFILENAME;
"! !

!OPENFILENAME methodsFor: 'constants' stamp: 'eem 11/6/2007 15:31'!
MAXPATH
	^260! !

!OPENFILENAME methodsFor: 'accessing' stamp: 'eem 2/19/2008 11:15'!
fileName
	^(Alien forPointer: (self unsignedLongAt: self lpstrFileOffset)) strcpyUTF8! !

!OPENFILENAME methodsFor: 'memory management' stamp: 'eem 2/21/2008 16:53'!
free
	self class primFree: (self unsignedLongAt:  self lpstrFileOffset).
	super free! !

!OPENFILENAME methodsFor: 'instance initalization' stamp: 'eem 11/6/2007 15:29'!
initialize
	self
		unsignedLongAt: self lStructSizeOffset put: self class dataSize;
		unsignedLongAt: self lpstrFileOffset put: (self class Ccalloc: self MAXPATH);
		unsignedLongAt: self nMaxFileOffset put: self MAXPATH! !

!OPENFILENAME methodsFor: 'field offsets' stamp: 'eem 11/11/2007 11:40'!
lStructSizeOffset
	^1 "0"! !

!OPENFILENAME methodsFor: 'field offsets' stamp: 'eem 11/11/2007 11:40'!
lpstrFileOffset
	^29 "28"! !

!OPENFILENAME methodsFor: 'field offsets' stamp: 'eem 11/11/2007 11:40'!
nMaxFileOffset
	^33 "32"! !

Alien variableByteSubclass: #VMCallbackContext32
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alien-Core'!

!VMCallbackContext32 commentStamp: 'eem 10/14/2010 14:02' prior: 0!
A VMCallbackContext32 is anAlien for the 32-bit version of VMCallbackContext (defined in the VMMaker package), a struct that maintains all the necessary context for a callback.

typedef struct {
    void *thunkp;
    char *stackptr;
    long *intRegArgs;
    double *floatRegArgs;
    void *savedCStackPointer;
    void *savedCFramePointer;
    union {
                            long vallong;
                            struct { int low, high; } valleint64;
                            struct { int high, low; } valbeint64;
                            double valflt64;
                            struct { void *addr; long size; } valstruct;
                        }   rvs;
    jmp_buf trampoline;
 } VMCallbackContext;

Instance Variables
!

!VMCallbackContext32 class methodsFor: 'instance creation' stamp: 'eem 10/14/2010 14:01'!
dataSize
	"We only access the fields before trampoline, so this is fine."
	^32! !

!VMCallbackContext32 methodsFor: 'accessing' stamp: 'eem 10/12/2010 17:40'!
floatResult: aFloat
	^self doubleAt: 25 put: aFloat! !

!VMCallbackContext32 methodsFor: 'accessing' stamp: 'eem 10/15/2010 10:12'!
floatregargsp
	^Alien forPointer: (self unsignedLongAt: 13)! !

!VMCallbackContext32 methodsFor: 'accessing' stamp: 'eem 10/15/2010 10:12'!
intregargsp
	^Alien forPointer: (self unsignedLongAt: 9)! !

!VMCallbackContext32 methodsFor: 'primitives' stamp: 'eem 10/15/2010 10:55'!
primReturnAs: typeCode "<SmallInteger>" fromContext: context "<MethodContext>"
	<primitive: 'primReturnAsFromContextThrough' module: 'IA32ABI' error: ec>
	^self primitiveFailed! !

!VMCallbackContext32 methodsFor: 'accessing' stamp: 'eem 10/14/2010 21:59'!
stackp
	^Alien forPointer: (self unsignedLongAt: 5)! !

!VMCallbackContext32 methodsFor: 'accessing' stamp: 'eem 10/12/2010 17:36'!
thunkp
	^self unsignedLongAt: 1! !

!VMCallbackContext32 methodsFor: 'accessing' stamp: 'eem 10/15/2010 10:14'!
wordResult: anInteger
	self signedLongAt: 25 put: anInteger.
	^1! !

!VMCallbackContext32 methodsFor: 'accessing' stamp: 'eem 10/14/2010 13:47'!
wordSize
	^4! !

Alien variableByteSubclass: #VMCallbackContext64
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alien-Core'!

!VMCallbackContext64 commentStamp: 'eem 10/14/2010 14:02' prior: 0!
A VMCallbackContext64 is anAlien for the 64-bit version of VMCallbackContext (defined in the VMMaker package), a struct that maintains all the necessary context for a callback.

typedef struct {
    void *thunkp;
    char *stackptr;
    long *intRegArgs;
    double *floatRegArgs;
    void *savedCStackPointer;
    void *savedCFramePointer;
    union {
                            long vallong;
                            struct { int low, high; } valleint64;
                            struct { int high, low; } valbeint64;
                            double valflt64;
                            struct { void *addr; long size; } valstruct;
                        }   rvs;
    jmp_buf trampoline;
 } VMCallbackContext;

Instance Variables
!

!VMCallbackContext64 class methodsFor: 'instance creation' stamp: 'eem 10/14/2010 14:01'!
dataSize
	"We only access the fields before trampoline, so this is fine."
	^64! !

!VMCallbackContext64 methodsFor: 'accessing' stamp: 'eem 10/15/2010 10:14'!
floatResult: aFloat
	self doubleAt: 49 put: aFloat.
	^3! !

!VMCallbackContext64 methodsFor: 'accessing' stamp: 'eem 10/15/2010 10:13'!
floatregargsp
	^Alien forPointer: (self unsignedLongAt: 25)! !

!VMCallbackContext64 methodsFor: 'accessing' stamp: 'eem 10/15/2010 10:13'!
integerResult: anInteger
	self signedLongLongAt: 49 put: anInteger.
	^1! !

!VMCallbackContext64 methodsFor: 'accessing' stamp: 'eem 10/15/2010 10:13'!
intregargsp
	^Alien forPointer: (self unsignedLongAt: 17)! !

!VMCallbackContext64 methodsFor: 'primitives' stamp: 'eem 5/13/2011 09:36'!
primReturnAs: typeCode "<SmallInteger>" fromContext: context "<MethodContext>"
	<primitive: 'primReturnAsFromContextThrough' module: 'IA32ABI' error: ec>
	^self primitiveFailed! !

!VMCallbackContext64 methodsFor: 'accessing' stamp: 'eem 10/15/2010 10:13'!
stackp
	^Alien forPointer: (self unsignedLongLongAt: 9)! !

!VMCallbackContext64 methodsFor: 'accessing' stamp: 'eem 10/12/2010 17:35'!
thunkp
	^self unsignedLongLongAt: 1! !

!VMCallbackContext64 methodsFor: 'accessing' stamp: 'eem 10/14/2010 13:47'!
wordSize
	^8! !

!ByteArray methodsFor: '*Alien-Core' stamp: 'eem 1/25/2008 14:56'!
utf8ToIso
	| s i v1 v2 v3 v4 |
	s := WriteStream on: (String new: self size).
	i := 1.
	[i <= self size] whileTrue:
		[v1 := self at: i.
		 i := i + 1.
		 v1 < 128
			ifTrue: [ s nextPut: v1 asCharacter ]
			ifFalse:
				[(i > self size) ifTrue:
					[self error: 'illegal end-of-string, expected 2nd byte of UTF-8'].
				 v2 := self at: i.
				 i := i + 1.
				 v1 < 224
					ifTrue:
						[s nextPut: ((v2 bitAnd: 63) bitOr: ((v1 bitAnd: 31) bitShift: 6)) asCharacter]
					ifFalse:
						[(i > self size) ifTrue:
							[self error: 'illegal end-of-string, expected 3rd byte of UTF-8'].
						 v3 := self at: i.
						 i := i + 1.
						 v1 < 240
							ifTrue:
								[s nextPut: ((v3 bitAnd: 63) bitOr: (((v2 bitAnd: 63) bitShift: 6) bitOr: ((v1 bitAnd: 31) bitShift: 12))) asCharacter]
							ifFalse:
								[(i > self size) ifTrue:
							[self error: 'illegal end-of-string, expected 4th byte of UTF-8'].
							 v4 := self at: i.
							 i := i + 1.
							 v1 < 248
									ifTrue: [s nextPut: ((v4 bitAnd: 63) bitOr: (((v3 bitAnd: 63) bitShift: 6) bitOr: (((v2 bitAnd: 63) bitShift: 12) bitOr: ((v1 bitAnd: 15) bitShift: 18)))) asCharacter]
									ifFalse:
										[self error: 'illegal UTF-8 character']]]]].
	^s contents! !

Error subclass: #NotFoundError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alien-Core'!

Object subclass: #AlienWeakTable
	instanceVariableNames: 'accessProtect firstUnusedIndex lastUsedIndex weakArray strongArray owner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alien-Core'!

!AlienWeakTable commentStamp: '<historical>' prior: 0!
This class supports simple post-mortem finalization of values associated with gc'ed objects.  An object to be finalized is registered in the table together with another object called 'the tag'. The finalizable object is held onto by the table weakly, the tag object--strongly. A table is initialized with the owner object, which is the object that performs the actual finalization. Some time after a finalizable object is garbage-collected, the owner is sent the #finalize: message with the object's tag as the argument.

Instance Variables:
	accessProtect <Semaphore>  - A mutex protecting state
	firstUnusedIndex <Integer> - The lowest index in strongArray that is empty (an invariant)
	lastUsedIndex <Integer> - The highest index in strongArray that is not empty (an invariant)
	weakArray <WeakArray> - The array of objects whose death we're interested in.
	strongArray <Array> - The array of corresponding objects that wll be passed to the owner when their corresponding element in weakArray is garbage collected.
	owner <Object> - The object that is sent finalize: with the tag of an object that has been garbage-collected.!

!AlienWeakTable class methodsFor: 'instance creation' stamp: 'eem 12/21/2007 15:00'!
newForOwner: anOwner
	^self basicNew initializeWithOwner: anOwner! !

!AlienWeakTable methodsFor: 'adding' stamp: 'vb 5/7/2008 08:34'!
add: anObject finalizing: postMortemFinalizationTag
	"Register a new object in the table, with the associated tag. The object is held onto weakly, the tag--strongly. The tag will be passed to the owner as the argument of #finalize: some time after the object is garbage collected."

	| size start delta |
	accessProtect critical:
		[size := strongArray size.
		 start := firstUnusedIndex].
	start to: size do:
		[:i|
		(strongArray at: i) == nil ifTrue:
			[accessProtect critical:
				[weakArray at: i put: anObject.
				 strongArray at: i put: postMortemFinalizationTag.
				 firstUnusedIndex := i + 1.
				 i > lastUsedIndex ifTrue: [lastUsedIndex := i]].
			 ^anObject]].
	delta := size min: 4096.
	accessProtect critical:
		[(weakArray := weakArray copyGrownBy: delta)
			 at: size + 1 put: anObject.
		 (strongArray := strongArray  copyGrownBy: delta)
			at: size put: postMortemFinalizationTag.
		 firstUnusedIndex := size + 1.
		 lastUsedIndex := size].
	^anObject! !

!AlienWeakTable methodsFor: 'finalization' stamp: 'vb 5/7/2008 08:35'!
finalizeValues
	"Sent periodically by the finalization loop to check for any dead objects and report their death to the owner."
	
	accessProtect critical:
		[| usedIndex value newLastUsedIndex |
		 newLastUsedIndex := 0.
		 1 to: lastUsedIndex do:
			[:i|
			(value := strongArray at: i) ifNotNil:
				[(weakArray at: i)
					ifNil:
						[strongArray at: i put: nil.
						 owner finalize: value.
						 firstUnusedIndex > i ifTrue:
							[firstUnusedIndex := i]]
					ifNotNil: [newLastUsedIndex := i]]].
		 lastUsedIndex := newLastUsedIndex]! !

!AlienWeakTable methodsFor: 'initialize-release' stamp: 'vb 5/8/2008 11:32'!
initializeWithOwner: anObject
	accessProtect := Semaphore forMutualExclusion.
	firstUnusedIndex := 1.
	lastUsedIndex := 0.
	weakArray := WeakArray new: 1024.
	strongArray := Array new: 1024.
	owner := anObject.
	WeakArray addWeakDependent: self! !

!AlienWeakTable methodsFor: 'removing' stamp: 'eem 1/11/2008 12:10'!
remove: anObject
	1 to: lastUsedIndex do:
		[:i|
		(weakArray at: i) == anObject ifTrue:
			[accessProtect critical:
				[strongArray at: i put: nil.
				 firstUnusedIndex > i ifTrue:
					[firstUnusedIndex := i].
				 i = lastUsedIndex ifTrue:
					[lastUsedIndex := i - 1]].
			 ^anObject]].
	^anObject! !

!AlienWeakTable methodsFor: 'removing' stamp: 'vb 5/8/2008 11:31'!
removeAll

	accessProtect critical:
		[1 to: lastUsedIndex do:
			[:index |
			strongArray at: index put: nil.
			weakArray at: index put: nil].
		firstUnusedIndex := 1.
		lastUsedIndex := 0]! !

Object subclass: #Callback
	instanceVariableNames: 'block thunk argsProxyClass evaluator numEvaluatorArgs'
	classVariableNames: 'ABI FlushThunkToCallbackMap ThunkToCallbackMap'
	poolDictionaries: ''
	category: 'Alien-Core'!

!Callback commentStamp: 'eem 11/26/2007 15:37' prior: 0!
Callbacks encapsulate callbacks from the outside world.  They allow Smalltalk blocks to be evaluated and answer their results to external (e.g. C) callees.

Instance Variables:
block <BlockContext> - The Smalltalk code to be run in response to external code invoking the callback.
thunk <FFICallbackThunk> - the wrapper around the machine-code thunk that initiates the callback and whose address should be passed to C
argsProxy <Alien> - the wrapper around the thunk's incomming stack pointer, used to extract arguments from the stack.
resultProxy <FFICalbackReturnValue> - the specification of the block's return value and its C type so that the result can be passed back in the right low-level form.

Class Variables:
ThunkToCallbackMap <Dictionary of: thunkAddress <Integer> -> callback <Callback>> - used to lookup the Callback associated with a specific thunk address on callback.  See FFICallbackThunk.!

!Callback class methodsFor: 'instance creation' stamp: 'eem 10/15/2010 09:25'!
block: aBlock "<BlockContext>" argsClass: argsProxyClass "<Alien class>" "^<Callback>"
	"Answer a Callback that evaluates aBlock and whose thunk should conform to the C calling convention."
	<legacy>
	^self new
		block: aBlock
		thunk: FFICallbackThunk newCCall
		argsProxyClass: argsProxyClass! !

!Callback class methodsFor: 'instance creation' stamp: 'eem 10/15/2010 09:25'!
block: aBlock "<BlockContext>" stdcallArgsClass: argsProxyClass "<Alien class>" "^<Callback>"
	"Answer a Callback that evaluates aBlock and whose thunk should conform to the
	 stdcall (Pascal) calling convention with a stack size of argsProxyClass dataSize."
	<legacy>
	^self new
		block: aBlock
		thunk: (FFICallbackThunk newStdCall: argsProxyClass dataSize)
		argsProxyClass: argsProxyClass! !

!Callback class methodsFor: 'evaluation' stamp: 'eem 10/17/2010 15:06'!
evaluateCallbackForContext: callbackContext "<VMCallbackContext32|VMCallbackContext64> ^<FFICallbackReturnValue>"
	(ThunkToCallbackMap
		at: callbackContext thunkp
		ifAbsent: [^self error: 'could not locate Callback instance corresponding to thunk address'])
		ifNil: [self error: 'Callback instance for this thunk address has been garbage collected']
		ifNotNil:
			[:callback|
			^callback valueInContext: callbackContext]! !

!Callback class methodsFor: 'evaluation' stamp: 'eem 10/14/2010 12:23'!
evaluateCallbackForThunk: thunkPtr "<Integer>" stack: stackPtr "<Integer>" "^<FFICallbackReturnValue>"
	<legacy>
	| callback |
	callback := ThunkToCallbackMap
		at: thunkPtr
		ifAbsent: [^self error: 'could not locate Callback instance corresponding to thunk address'].
	callback isNil
		ifTrue: [self error: 'Callback instance for this thunk address has been garbage collected']
		ifFalse: [^callback valueWithStackPointer: stackPtr]! !

!Callback class methodsFor: 'finalization' stamp: 'eem 2/20/2008 16:11'!
finalizeValues
	FlushThunkToCallbackMap := true! !

!Callback class methodsFor: 'class initialization' stamp: 'eem 10/15/2010 09:42'!
initialize
	"Callback initialize"
	[self setup: false] "Monticello... grrr...."
		on: Error
		do: [:ex| ]! !

!Callback class methodsFor: 'class initialization' stamp: 'eem 10/15/2010 09:42'!
setup: resuming
	"Callback setup"
	| prevMap |
	ABI := 'IA32'. "for now; value needs to be answered by a primitive or system attribute etc"
	FlushThunkToCallbackMap := false.
	(WeakArray hasWeakDependent: self) ifFalse:
		[WeakArray addWeakDependent: self].
	prevMap := resuming ifFalse: [ThunkToCallbackMap].
	ThunkToCallbackMap := WeakValueDictionary new.
	prevMap ifNotNil:
		[prevMap keysAndValuesDo:
			[:thunkAddress :callback|
			ThunkToCallbackMap at: thunkAddress put: callback]]! !

!Callback class methodsFor: 'instance creation' stamp: 'eem 10/15/2010 10:41'!
signature: signature "<String>" block: aBlock "<BlockClosure> ^<Callback>"
	"Answer a Callback that evaluates aBlock whose C signature is signature 
	 and whose thunk should conform to the C calling convention."
	^self new signature: signature block: aBlock! !

!Callback methodsFor: 'private' stamp: 'eem 10/15/2010 10:50'!
addToThunkTable
	FlushThunkToCallbackMap ifTrue:
		[FlushThunkToCallbackMap := false.
		 ThunkToCallbackMap keys do:
			[:k| (ThunkToCallbackMap at: k) ifNil: [ThunkToCallbackMap removeKey: k]]].
	ThunkToCallbackMap at: thunk addressField put: self! !

!Callback methodsFor: 'instance initialization' stamp: 'eem 10/15/2010 10:51'!
block: aBlock "<BlockClosure>" thunk: thunkWrapper "<FFICallbackThunk>" argsProxyClass: argsWrapperClass "<Alien class>"
	<legacy>
	block := aBlock.
	thunk := thunkWrapper.
	argsProxyClass := argsWrapperClass.
	self addToThunkTable! !

!Callback methodsFor: 'private' stamp: 'eem 10/15/2010 10:49'!
evaluatorForSignature: signature "<String>"

	Pragma withPragmasIn: self class do:
		[:pragma|
		(pragma key == #signature:abi:
		and: [(pragma argumentAt: 2) = ABI
		and: [(pragma argumentAt: 1) = signature]]) ifTrue:
			[^pragma method]].
	self error: 'could not find Callback signature'! !

!Callback methodsFor: 'signatures' stamp: 'eem 11/1/2010 16:40'!
intcharstarRetint: callbackContext sp: spAlien
	<signature: 'int (*)(int, char *)' abi: 'IA32'>
	^callbackContext wordResult:
		((block
			value: (spAlien signedLongAt: 1) ~= 0
			value: (Alien forPointer: (spAlien unsignedLongAt: 5)) strcpyUTF8)
				ifNil: [0]
				ifNotNil:
					[:result|
					result isInteger
						ifTrue: [result]
						ifFalse: [result == true ifTrue: [1] ifFalse: [0]]])! !

!Callback methodsFor: 'instance initialization' stamp: 'eem 10/15/2010 10:51'!
signature: signature "<String>" block: aBlock "<BlockClosure>"
	| method |
	method := self evaluatorForSignature: signature.
	block := aBlock.
	thunk := (method pragmaAt: #calleepops:)
				ifNil: [FFICallbackThunk newCCall]
				ifNotNil:
					[:pragma|
					FFICallbackThunk newStdCall: (pragma argumentAt: 1)].
	numEvaluatorArgs := (evaluator := method selector) numArgs.
	self addToThunkTable! !

!Callback methodsFor: 'accessing' stamp: 'eem 11/19/2007 16:54'!
thunk
	^thunk! !

!Callback methodsFor: 'evaluation' stamp: 'eem 10/15/2010 10:10'!
valueInContext: callbackContext "<VMCallbackContext32|VMCallbackContext64> ^<Integer>"
	numEvaluatorArgs = 2 ifTrue:
		[^self perform: evaluator
			with: callbackContext
			with: callbackContext stackp].
	numEvaluatorArgs = 3 ifTrue:
		[^self perform: evaluator
			with: callbackContext
			with: callbackContext stackp
			with: callbackContext intregargsp].
	numEvaluatorArgs = 4 ifTrue:
		[^self perform: evaluator
			with: callbackContext
			with: callbackContext stackp
			with: callbackContext intregargsp
			with: callbackContext floatregargsp].
	self error: 'invalid evaluator argument count'! !

!Callback methodsFor: 'evaluation' stamp: 'eem 10/14/2010 12:23'!
valueWithStackPointer: stackPtr "<Integer>" "^<FFICallbackReturnValue>"
	<legacy>
	| resultProxy |
	block
		value: (argsProxyClass atAddress: stackPtr)
		value: (resultProxy := FFICallbackReturnValue new).
	^resultProxy! !

!Callback methodsFor: 'signatures' stamp: 'eem 11/1/2010 11:49'!
voidstarvoidstarRetint: callbackContext sp: spAlien
	<signature: 'int (*)(const void *, const void *)' abi: 'IA32'>
	^callbackContext wordResult:
		(block
			value: (Alien forPointer: (spAlien unsignedLongAt: 1))
			value: (Alien forPointer: (spAlien unsignedLongAt: 5)))! !

Object subclass: #UnsafeAlien
	instanceVariableNames: 'nonPointerObject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alien-Core'!

!UnsafeAlien commentStamp: '<historical>' prior: 0!
Instances of UnsafeAlien represent the addresses of heap-resident non-pointer Smalltalk objects as actual parameters in FFI call-outs.  An UnsafeAlien on (e.g.) a ByteString used as a parameter in an FFI call causes the FFI machinery to pass the address of the first byte in the ByteString.  THIS IS UNSAFE!!  It is unsafe because
a) the garbage collector can potentially move the ByteString (or any other object) during the call, because the call may call-back, invoking the garbage collector,
b) if external code retains the address for longer than the duration of the call and dereferences it in a subsequent call the object may have moved in the mean time,
c) the address of the object is passed without any other potentially necessary conversions such as null-termination
d) the hundred other problems this benighted author hasn't thought of.
Hence UnsafeAlien is to be used carefully by clients that know that the usage is safe.
You have been warned ;)

Create instances via
	UnsafeAlien forPointerTo: 'You are on your own!!', (ByteString with: (Character value: 0))!

!UnsafeAlien class methodsFor: 'class initialization' stamp: 'eem 5/22/2010 14:59'!
ensureInSpecialObjectsArray
	"UnsafeAlien must be in the specialObjectsArray to enable the Newsqueak FFI.
	 Because of the bootstrap the specialObjectsArray may already have been
	 partially initialized with the callback selector and to be large enough (as part of
	 loading immutability), but it will be missing UnsafeAlien until UnsafeAlien is loaded.
	 So check if the specialObjectsArray is of the expected size before recreating
	 and slam UnsafeAlien in if the specialObjectsArray is already large enough."

	| index |
	self ~~ UnsafeAlien ifTrue: [^self].

	index := 55.
	((Smalltalk includesKey: #ObjectMemory)
	 and: [((Smalltalk at: #ObjectMemory) classPool at: #ClassUnsafeAlien ifAbsent: []) ~~ (index - 1)]) ifTrue:
		[self error: 'index probably wrong'].

	Smalltalk specialObjectsArray size < index ifTrue:
		[Smalltalk recreateSpecialObjectsArray].
	
	(Smalltalk specialObjectsArray size < index
	 or: [(Smalltalk specialObjectsArray at: index) ~~ nil
		and: [(Smalltalk specialObjectsArray at: index) ~~ self]]) ifTrue:
		[self error: 'SystemDictionary>>recreateSpecialObjectsArray appears incompatible'].

	(Smalltalk specialObjectsArray at: index) == nil ifTrue:
		[Smalltalk specialObjectsArray at: index put: self]! !

!UnsafeAlien class methodsFor: 'examples' stamp: 'eem 4/18/2008 15:41'!
exampleSprintf
	"Call the libc sprintf function."
	"UnsafeAlien exampleSprintf"
	| r s |
	(Alien lookup: 'sprintf' inLibrary: Alien libcName)
		primFFICallResult: (r := Alien new: 4)
		withArguments: {self forPointerTo: (s := ByteString new: 1024).
						 self forPointerTo: 'Hello World %d %x !!!!', (String with: Character lf).
						 123.
						 48879}.
	^s copyFrom: 1 to: (r signedLongAt: 1) - 1 "trim the null byte"! !

!UnsafeAlien class methodsFor: 'instance creation' stamp: 'eem 12/4/2008 12:54'!
forPointerTo: anObject
	(anObject class instSpec between: 6 and: 8) ifFalse:
		[^self error: 'only use UnsafeAlien on instances of isBits classes (other than CompiledMethod)!!'].
	^self new nonPointerObject: anObject! !

!UnsafeAlien class methodsFor: 'class initialization' stamp: 'eem 4/18/2008 15:15'!
initialize
	"UnsafeAlien initialize"
	self ensureInSpecialObjectsArray! !

!UnsafeAlien methodsFor: 'accessing' stamp: 'eem 4/18/2008 14:38'!
object
	^nonPointerObject! !

TestCase subclass: #AlienSunit
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alien-CoreTest'!

!AlienSunit methodsFor: 'testing' stamp: 'John M McIntosh 11/23/2008 21:57'!
testAccessing
	| alien |
	alien := Alien newCString: 'foo'.
	alien freeAfter: [:cstring | 
			self should: [cstring dataSize = 4].
			self should: [cstring strcpy = 'foo']].
	self should: [alien address = 0].
	self should: [alien dataSize = 0].! !

!AlienSunit methodsFor: 'testing' stamp: 'John M McIntosh 11/23/2008 23:43'!
testAtAddress
	| alien address alias |
	alien := Alien newCString: 'foo'.
	address := alien address.
	"This means alias and alien both point to the same address"
	alias := Alien atAddress: address dataSize: 4.
	self should: [alias dataSize = 4].
	self should: [alias strcpy = 'foo'].
	alien free.! !

!AlienSunit methodsFor: 'testing' stamp: 'John M McIntosh 11/25/2008 22:08'!
testByte
	| byte |
	byte := Alien newC:1.
	0 to: 255 do: 
		[:i  | byte unsignedByteAt: 1 put: i.
		self should: [(byte unsignedByteAt: 1) = i]].
	-128 to: 127 do: 
		[:j |  byte signedByteAt: 1 put: j.
		self should: [(byte signedByteAt: 1) = j]].
	-255 to: -129 do: 
		[:k |  self should: [byte signedByteAt: 1 put: k] raise: Error].
	128 to: 255 do: 
		[:k |  self should: [byte signedByteAt: 1 put: k] raise: Error].! !

!AlienSunit methodsFor: 'testing' stamp: 'John M McIntosh 11/23/2008 23:44'!
testByteAccessors
	| alien |
	alien := Alien newC: 2. 
	alien signedByteAt: 1 put: 16r7F.
	self should: [(alien signedByteAt: 1) = 16r7F].
	self should: [(alien asSignedByte) = 16r7F].
	self should: [alien signedByteAt: 1 put: 16r80] raise: Error.
	alien signedByteAt: 2 put: 16r00.
	alien signedByteAt: 1 put: 16r7F.
	self should: [(alien signedByteAt: 2) = 16r00].
	alien signedByteAt: 1 put: -1*16r80.
	self should: [(alien signedByteAt: 1) =  (-1*16r80)].
	self should: [alien signedByteAt: 1 put: -1*16r81] raise: Error.
	alien unsignedByteAt: 1 put: 16r7F.
	self should: [(alien signedByteAt: 1) = 16r7F].
	alien unsignedByteAt: 1 put: 16r80.
	self should: [(alien asSignedByte) = -128].
	self should: [(alien signedByteAt: 1) = -128].
	alien unsignedByteAt: 1 put: 16r81.
	self should: [(alien signedByteAt: 1) = -127].
	alien unsignedByteAt: 1 put: 16r7F.
	self should: [(alien unsignedByteAt: 1) = 16r7F].
	alien unsignedByteAt: 1 put: 16r80.
	self should: [(alien unsignedByteAt: 1) = 16r80].
	alien unsignedByteAt: 1 put: 16r81.
	self should: [(alien unsignedByteAt: 1) = 16r81].
	alien free.! !

!AlienSunit methodsFor: 'testing' stamp: 'John M McIntosh 11/23/2008 23:44'!
testDoubleAccessors
	| alien |
	alien := Alien newC: 8. 
	alien doubleAt: 1 put: 8.8.
	self should: [(alien doubleAt: 1) = 8.8].
		alien free.! !

!AlienSunit methodsFor: 'testing' stamp: 'John M McIntosh 11/23/2008 23:44'!
testFloatAccessors
	| alien |
	alien := Alien newC: 4. 
	alien floatAt: 1 put: 8.8.
	self should: [(alien floatAt: 1) closeTo: 8.8].
		alien free.! !

!AlienSunit methodsFor: 'testing' stamp: 'John M McIntosh 11/25/2008 22:09'!
testLong
	| long unsignedIntegerIntervalue k signedIntegerIntervalue |
	
	long := Alien newC: 4.
	#( 0 4294967295) do: 
		[:i  | long unsignedLongAt: 1 put: i.
		self should: [(long unsignedLongAt: 1) = i]].
	#( -2147483648 2147483647) do: 
		[:j |  long signedLongAt: 1 put: j.
		self should: [(long signedLongAt: 1) = j]].
	unsignedIntegerIntervalue := (0 to: 4294967295).
	10000 timesRepeat: [
			k := unsignedIntegerIntervalue atRandom.
			long unsignedLongAt: 1 put: k.
			self should: [(long unsignedLongAt: 1) = k]].
	signedIntegerIntervalue := (-2147483648 to: 2147483647).
	10000 timesRepeat: [
			k := signedIntegerIntervalue atRandom.
			long signedLongAt: 1 put: k.
			self should: [(long signedLongAt: 1) = k]].
	
	signedIntegerIntervalue := (-4294967295 to: -2147483649).
	10000 timesRepeat: [
		k := signedIntegerIntervalue atRandom.
		self should: [long signedLongAt: 1 put: k] raise: Error].
	
	signedIntegerIntervalue := (2147483648 to: 4294967295).
	10000 timesRepeat: [
		k := signedIntegerIntervalue atRandom.
		self should: [long signedLongAt: 1 put: k] raise: Error].
	! !

!AlienSunit methodsFor: 'testing' stamp: 'John M McIntosh 11/23/2008 23:44'!
testLongAccessors
	| alien |
	Smalltalk wordSize = 4 ifFalse: [^self].
	
	alien := Alien newC: 4. 
	alien signedLongAt: 1 put: 16r7FFFFFFF.
	self should: [(alien signedLongAt: 1) = 16r7FFFFFFF].
	self should: [(alien asSignedLong) = 16r7FFFFFFF].
	self should: [alien signedLongAt: 1 put: 16r80000000] raise: Error.
	alien signedLongAt: 1 put: -1*16r80000000.
	self should: [(alien signedLongAt: 1) =  (-1*16r80000000)].
	self should: [alien signedLongAt: 1 put: -1*16r80000001] raise: Error.
	alien unsignedLongAt: 1 put: 16r7FFFFFFF.
	self should: [(alien signedLongAt: 1) = 16r7FFFFFFF].
	alien unsignedLongAt: 1 put: 16r80000000.
	self should: [(alien signedLongAt: 1) = -2147483648].
	alien unsignedLongAt: 1 put: 16r80000001.
	self should: [(alien signedLongAt: 1) = -2147483647].
	alien unsignedLongAt: 1 put: 16r7FFFFFFF.
	self should: [(alien unsignedLongAt: 1) = 16r7FFFFFFF]. 
	self should: [(alien asUnsignedLong) = 16r7FFFFFFF]. 
	alien unsignedLongAt: 1 put: 16r80000000.
	self should: [(alien unsignedLongAt: 1) = 16r80000000].
	alien unsignedLongAt: 1 put: 16r80000001.
	self should: [(alien unsignedLongAt: 1) = 16r80000001].
		alien free.! !

!AlienSunit methodsFor: 'testing' stamp: 'John M McIntosh 11/25/2008 22:09'!
testLongLong
	| unsignedIntegerIntervalue k signedIntegerIntervalue longlong |

	longlong := Alien newC: 8.
	#( 0 4294967295 18446744073709551615) do: 
		[:i  | longlong unsignedLongLongAt: 1 put: i.
		self should: [(longlong unsignedLongLongAt: 1) = i]].
	#(-9223372036854775808  -2147483648 2147483647 9223372036854775807) do: 
		[:j |  longlong signedLongLongAt: 1 put: j.
		self should: [(longlong signedLongLongAt: 1) = j]].
	unsignedIntegerIntervalue := (0 to: 18446744073709551615).
	10000 timesRepeat: [
			k := unsignedIntegerIntervalue atRandom.
			longlong unsignedLongLongAt: 1 put: k.
			self should: [(longlong unsignedLongLongAt: 1) = k]].
	signedIntegerIntervalue := (-9223372036854775808 to: 9223372036854775807).
	10000 timesRepeat: [
			k := signedIntegerIntervalue atRandom.
			longlong signedLongLongAt: 1 put: k.
			self should: [(longlong signedLongLongAt: 1) = k]].
	
	signedIntegerIntervalue := (-18446744073709551615 to: -9223372036854775809).
	10000 timesRepeat: [
		k := signedIntegerIntervalue atRandom.
		self should: [longlong signedLongLongAt: 1 put: k] raise: Error].
	
	signedIntegerIntervalue := (9223372036854775808 to: 18446744073709551615).
	10000 timesRepeat: [
		k := signedIntegerIntervalue atRandom.
		self should: [longlong signedLongLongAt: 1 put: k] raise: Error].
	! !

!AlienSunit methodsFor: 'testing' stamp: 'eem 12/2/2008 10:51'!
testParser
	"Just confirm it will parse"
	
	(Parser new parse: (MethodNode sourceCodeAt: #primitiveErrorVariableName) class: MethodNode) primitiveErrorVariableName.
	(Parser new parse: 'foo <primitive: 111 error: ''foo''> self primitiveFailed' class: Object) primitiveErrorVariableName.
	(Parser new parse: 'foo <primitive: 111 error: foo> self primitiveFailed' class: Object) primitiveErrorVariableName.
	(Parser new parse: 'foo <primitive: 111> self primitiveFailed' class: Object) primitiveErrorVariableName.
	(Parser new parse: 'foo <primitive: 111 error: foo> self primitiveFailed' class: Object) generate.
! !

!AlienSunit methodsFor: 'testing' stamp: 'John M McIntosh 11/23/2008 22:00'!
testPointer
	| alien address alias |
	alien := Alien newC: 3.
	address := alien address.
	alias := Alien forPointer: address.
	self should: [alias dataSize = 0].
	self should: [alias address = address].
	alien free.! !

!AlienSunit methodsFor: 'testing' stamp: 'John M McIntosh 11/25/2008 22:09'!
testShort
	| short |
	short := Alien new: 2.
	0 to: 65535 do: 
		[:i  | short unsignedShortAt: 1 put: i.
		self should: [(short unsignedShortAt: 1) = i]].
	-32768 to: 32767 do: 
		[:j |  short signedShortAt: 1 put: j.
		self should: [(short signedShortAt: 1) = j]].
	-65535 to: -32769 do: 
		[:k |  self should: [short signedShortAt: 1 put: k] raise: Error].
	32768 to: 65535 do: 
		[:k |  self should: [short signedShortAt: 1 put: k] raise: Error].! !

!AlienSunit methodsFor: 'testing' stamp: 'John M McIntosh 11/23/2008 23:44'!
testShortAccessors
	| alien |
	alien := Alien newC: 2. 
	alien signedShortAt: 1 put: 16r7FFF.
	self should: [(alien signedShortAt: 1) = 16r7FFF].
	self should: [(alien asSignedShort) = 16r7FFF].
	self should: [alien signedShortAt: 1 put: 16r8000] raise: Error.
	alien signedShortAt: 1 put: -1*16r8000.
	self should: [(alien signedShortAt: 1) =  (-1*16r8000)].
	self should: [alien signedShortAt: 1 put: -1*16r8001] raise: Error.
	alien unsignedShortAt: 1 put: 16r7FFF.
	self should: [(alien signedShortAt: 1) = 16r7FFF].
	alien unsignedShortAt: 1 put: 16r8000.
	self should: [(alien signedShortAt: 1) = -32768].
	alien unsignedShortAt: 1 put: 16r8001.
	self should: [(alien signedShortAt: 1) = -32767].
	alien unsignedShortAt: 1 put: 16r7FFF.
	self should: [(alien unsignedShortAt: 1) = 16r7FFF].
	self should: [(alien asUnsignedShort) = 16r7FFF].
	alien unsignedShortAt: 1 put: 16r8000.
	self should: [(alien unsignedShortAt: 1) = 16r8000].
	alien unsignedShortAt: 1 put: 16r8001.
	self should: [(alien unsignedShortAt: 1) = 16r8001].
		alien free.! !

!AlienSunit methodsFor: 'testing' stamp: 'John M McIntosh 11/23/2008 22:00'!
testasAlien
	| alien alienValue |
	alien := Alien newCString: 'foo'.
	self should: [alien == alien asAlien].
	alien free.
	alienValue := false asAlien.
	self should: [alienValue isNumber and: [alienValue == 0]].
	alienValue := true asAlien.
	self should: [alienValue isNumber and: [alienValue == 1]].
	alienValue := nil asAlien.
	self should: [alienValue isNumber and: [alienValue == 0]].! !

!AlienSunit methodsFor: 'testing' stamp: 'John M McIntosh 11/23/2008 15:18'!
testnewC

	| alien |
	alien := 'foo' asAlien.
	self should: [alien strcpy = 'foo'].
	alien free. ! !

!AlienSunit methodsFor: 'testing' stamp: 'John M McIntosh 11/23/2008 23:43'!
teststrcpy

	| alien |
	alien := 'foo' asAlien.
	self should: [alien strcpy = 'foo'].
	self should: [(alien strcpyFrom: 2) = 'oo'].
	alien free.
	alien := 'foo' asAlien.
	self should: [alien strcpyUTF8 = 'foo'].
	self should: [(alien strcpyUTF8From: 2) = 'oo'].
	alien free.
! !

!AlienSunit methodsFor: 'testing' stamp: 'eem 5/10/2010 17:23'!
willCrashTheVMtestLibCExamples
	| sortedData unsorted sorted |
	
	sortedData := Alien exampleCqsort.
	unsorted := sortedData key copy.
	sorted := unsorted asSortedCollection.
	self should: [sortedData value = sorted asArray].
	Alien examplePrintf.! !

!False methodsFor: '*Alien-Core' stamp: 'John M McIntosh 11/17/2008 17:02'!
asAlien
	^ 0! !
Alien initialize!
FFICallbackThunk initialize!
Callback initialize!
UnsafeAlien initialize!
